//+------------------------------------------------------------------+
//|                                       Output History (indicator) |
//+------------------------------------------------------------------+

#property indicator_chart_window     

#include <hanover --- function header (np).mqh>

extern string    CurrencyPairs                     = "AC,AF,AJ,AN,AU,CF,CJ,FJ,EA,EC,EF,EG,EJ,EN,EU,GA,GC,GF,GJ,GN,GU,NC,NF,NJ,NU,UC,UF,UJ"; 
                                                                        // enter list of currency pairs, separated by commas (if blank, use current chart's pair)
extern string    OutputFileName                    = "Room Values output.csv";
extern string    HeaderLine                        = "Symbol,ADR,RmUp,RmDn,T,RmLmt";
extern string    FieldSeparator                    = ",";
extern bool      EncloseValuesInQuotes             = false;
//extern string    DateTimeFormat                    = "M/D/Y,H:I";
//extern string    PriceFormat                       = "T3.5";
//extern string    PriceFormatJPYpairs               = "T5.3";
//extern string    VolumeFormat                      = "T10";
extern string    OutputFormat                      = "T10";
extern string    RefreshPeriod                     = "+0";

extern bool      ExcludeSundayData                 = true;
extern double    RoomLimitRatio                    = 0.2;

int ATR1_Prd=5, ATR2_Prd=25;

string     P[40], Quote, IndiName, header;
int        handle, np, nt, RefreshEveryXMins, dig;
double     pnt, spr, tickval;
datetime   prev_time;
bool       FirstTime;

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+
  IndiName = NumberToStr(GetUniqueInt(),"'OutHist-'Z6'-'");

  if (StringLen(CurrencyPairs) < 1)    CurrencyPairs = Symbol();
  np = StrToStringArray(CurrencyPairs,P);
  Quote = "";
  if (EncloseValuesInQuotes)  Quote = "\x22";
  
  if (FieldSeparator == "^")     
    FieldSeparator = "\x09";
//  DateTimeFormat = StringReplace(DateTimeFormat,"^","\x09");

  RefreshEveryXMins = StrToTF(RefreshPeriod);
  prev_time = -9999;
  
  header = StringReplace(HeaderLine,",",FieldSeparator);
  
  del_obj();
  plot_obj();
  return(0);
}

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
//  MessageBox("History output complete","Output History");
  del_obj();
  return(0);
}

//+------------------------------------------------------------------+
void del_obj()  {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  if (RefreshEveryXMins < 0)  {
    if (FirstTime)  {
      del_obj();
      plot_obj();
    }
    FirstTime = false;      
    return(0);
  }  
  if (RefreshEveryXMins == 0) {
    del_obj();
    plot_obj();    
  } else {
    if (prev_time != iTime(Symbol(),RefreshEveryXMins,0))  {
      del_obj();
      plot_obj();
      prev_time = iTime(Symbol(),RefreshEveryXMins,0);
  } }      
  return(0);
}

//+------------------------------------------------------------------+
int plot_obj()  {
//+------------------------------------------------------------------+
  handle = FileOpen(OutputFileName, FILE_CSV|FILE_WRITE, ',');
  if (handle<1)    return(0);
  if (StringLen(HeaderLine)>0) 
    FileWrite(handle,header);      // modify header line if adding more columns

  for (int i=0; i<np; i++)  {
    out_hist(ExpandCcy(P[i]));
  }

  FileClose(handle);

  return(0);
}

//+------------------------------------------------------------------+
int out_hist (string ccy)  {
//+------------------------------------------------------------------+
  pnt     = MarketInfo(ccy,MODE_POINT);
  dig     = MarketInfo(ccy,MODE_DIGITS);
  spr     = MarketInfo(ccy,MODE_SPREAD);
  tickval = MarketInfo(ccy,MODE_TICKVALUE);
  if (dig == 3 || dig == 5) {
    pnt     *= 10;
    spr     /= 10;
    tickval *= 10;
  } 
  int atr1 = MathRound(iATR(ccy,0,ATR1_Prd,1)/pnt);
  int atr2 = MathRound(iATR(ccy,0,ATR2_Prd,1)/pnt);
  int n=1;  
  double s=0.0;    
  for (int i=1; i<=20; i++) {
    while(ExcludeSundayData && TimeDayOfWeek(iTime(ccy,PERIOD_D1,n))==0) n++;
    s = s + ( iHigh(ccy,PERIOD_D1,n) - iLow(ccy,PERIOD_D1,n) )/pnt;
    if (i==1)  int adr1  = MathRound(s);
    if (i==5)  int adr5  = MathRound(s/5);
    if (i==10) int adr10 = MathRound(s/10);
    if (i==20) int adr20 = MathRound(s/20);
    n++;  
  }  
  int    adr   = MathRound((adr1+adr5+adr10+adr20)/4.0);
  double low0  = iLow(ccy,PERIOD_D1,0);
  double high0 = iHigh(ccy,PERIOD_D1,0);
  int    t     = MathRound((high0 - low0)/pnt);  
  int    RmUp  = MathRound(adr - (iClose(ccy,PERIOD_D1,0) - low0)/pnt);
  int    RmDn  = MathRound(adr - (high0 - iClose(ccy,PERIOD_D1,0))/pnt);
  int    RmLmt = MathRound(adr*RoomLimitRatio);

//  string OHLCFormat = PriceFormat;
//  if (StringFind(ccy,"JPY") >= 0)
//    OHLCFormat = PriceFormatJPYpairs;
  FileWrite(handle,      Quote + ccy                                           + Quote
      + FieldSeparator + Quote + NumberToStr(adr,OutputFormat)                 + Quote
      + FieldSeparator + Quote + NumberToStr(RmUp,OutputFormat)                + Quote
      + FieldSeparator + Quote + NumberToStr(RmDn,OutputFormat)                + Quote
      + FieldSeparator + Quote + NumberToStr(t,OutputFormat)                   + Quote
      + FieldSeparator + Quote + NumberToStr(RmLmt,OutputFormat)               + Quote
    );

  return(0);
}

//+------------------------------------------------------------------+
#include <hanover --- extensible functions (np).mqh>